-------------------------------------------------------------------------------
-- Title        : SVEC System FPGA top level
-- Project      : Simple VME64x FMC Carrier (SVEC)
-------------------------------------------------------------------------------
-- File         : svec_sfpga_top.vhd
-- Author       : Tomasz Włostowski
-- Company      : CERN BE-CO-HT
-- Created      : 2012-03-20
-- Last update  : 2013-01-25
-- Platform     : FPGA-generic
-- Standard     : VHDL '93
-------------------------------------------------------------------------------
-- Description: Top level of the System FPGA. Contains a stripped-down VME64x
-- core and the Appliaction FPGA bootloader core. Used solely for booting up
-- the AFPGA. Possible boot configurations are: HOST -> AFPGA, FLASH -> AFPGA
-- and HOST -> FLASH.
-------------------------------------------------------------------------------
--
-- Copyright (c) 2013 CERN / BE-CO-HT
--
-- This source file is free software; you can redistribute it   
-- and/or modify it under the terms of the GNU Lesser General   
-- Public License as published by the Free Software Foundation; 
-- either version 2.1 of the License, or (at your option) any   
-- later version.                                               
--
-- This source is distributed in the hope that it will be       
-- useful, but WITHOUT ANY WARRANTY; without even the implied   
-- warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      
-- PURPOSE.  See the GNU Lesser General Public License for more 
-- details.                                                     
--
-- You should have received a copy of the GNU Lesser General    
-- Public License along with this source; if not, download it   
-- from http://www.gnu.org/licenses/lgpl-2.1.html
--
-------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;

use work.gencores_pkg.all;
use work.wishbone_pkg.all;
use work.svec_bootloader_pkg.all;
use work.vme64x_pkg.all;

library UNISIM;
use UNISIM.vcomponents.all;

entity svec7_sfpga_top is
  port
    (

      -------------------------------------------------------------------------
      -- Standard SVEC ports (Clocks & Reset)
      -------------------------------------------------------------------------

      lclk_i : in std_logic;          -- 20 MHz VCXO clock
      rst_n_i  : in std_logic;

      -------------------------------------------------------------------------
      -- VME Interface pins
      -------------------------------------------------------------------------

      vme_write_n_i    : in    std_logic;
      vme_sysclk_i : in std_logic;
      vme_sysreset_n_i : in    std_logic;
      vme_retry_oe_o   : inout   std_logic;
      vme_retry_n_o    : inout   std_logic;
      vme_lword_n_b    : inout std_logic;
      vme_iackout_n_o  : inout   std_logic;
      vme_iackin_n_i   : in    std_logic;
      vme_iack_n_i     : in    std_logic;
      vme_gap_i        : in    std_logic;
      vme_dtack_oe_o   : inout   std_logic;
      vme_dtack_n_o    : inout  std_logic;
      vme_ds_n_i       : in    std_logic_vector(1 downto 0);
      vme_data_oe_n_o  : inout   std_logic;
      vme_data_dir_o   : inout   std_logic;
      vme_berr_o       : inout   std_logic;
      vme_as_n_i       : in    std_logic;
      vme_addr_oe_n_o  : inout   std_logic;
      vme_addr_dir_o   : inout   std_logic;
      vme_irq_o        : inout   std_logic_vector(7 downto 1);
      vme_ga_i         : in    std_logic_vector(4 downto 0);
      vme_data_b       : inout std_logic_vector(31 downto 0);
      vme_am_i         : in    std_logic_vector(5 downto 0);
      vme_addr_b       : inout std_logic_vector(31 downto 1);

      vme_berr_r_i : in std_logic;
      vme_dtack_r_i : in std_logic;
      vme_retry_r_i : in std_logic;
      
      
      vme_trst_i : in std_logic;
      vme_tck_i : in std_logic;
      vme_tdi_i : in std_logic;
      vme_tms_i : in std_logic;
      vme_tdo_o : out std_logic;
      vme_tdo_oe_o : out std_logic;
      
      afpga_clk_p_i : in std_logic;
      afpga_clk_n_i : in std_logic;

      afpga_rx_i : in std_logic_vector(4 downto 0);
      afpga_tx_o : out std_logic_vector(4 downto 0);

      afpga_cso_b_b : inout std_logic;
      afpga_mosi_b : inout std_logic;
      afpga_miso_i : inout std_logic;
      afpga_cclk_b : inout std_logic;

      pushbutton_i :in std_logic;
      -------------------------------------------------------------------------
      -- AFPGA boot signals
      -------------------------------------------------------------------------

      boot_clk_b    : inout std_logic;
      boot_config_o : inout std_logic;
      boot_done_i   : in  std_logic;
      boot_dout_o   : inout std_logic;
      boot_status_i : in  std_logic;
      boot_cs_o : inout std_logic;
      boot_din_o : inout std_logic;

      -------------------------------------------------------------------------
      -- SPI Flash Interface
      -------------------------------------------------------------------------

      spi_sys_cs_n_o : out std_logic;
      spi_sys_mosi_o : out std_logic;
      spi_sys_miso_i : in  std_logic;
      spi_sys_sclk_o : out std_logic;

      debugled_n_o : out std_logic_vector(2 downto 1);

      afpga_mode_ctrl_o : inout std_logic_vector(2 downto 0);
      
      -- Onboard PLL enable signal. Must be one for the clock system to work.
      pll_ce_o : out std_logic

      );
end svec7_sfpga_top;

architecture rtl of svec7_sfpga_top is

  constant c_PLL_RESET_DURATION : integer := 300;

  component reset_gen
    port (
      clk_sys_i       : in  std_logic;
      rst_vme_n_a_i   : in  std_logic;
      rst_local_n_a_i : in  std_logic;
      rst_n_o         : out std_logic);
  end component;


  component chipscope_ila
    port (
      CONTROL : inout std_logic_vector(35 downto 0);
      CLK     : in    std_logic;
      TRIG0   : in    std_logic_vector(31 downto 0);
      TRIG1   : in    std_logic_vector(31 downto 0);
      TRIG2   : in    std_logic_vector(31 downto 0);
      TRIG3   : in    std_logic_vector(31 downto 0));
  end component;

  component chipscope_icon
    port (
      CONTROL0 : inout std_logic_vector (35 downto 0));
  end component;

  -- signal VME_DATA_o_int                    : std_logic_vector(31 downto 0);
  -- signal vme_dtack_oe_int, VME_DTACK_n_int : std_logic;
  -- signal VME_DATA_OE_N_int                 : std_logic;

  signal wb_vme_in  : t_wishbone_master_out;
  signal wb_vme_out : t_wishbone_master_in;

  signal passive : std_logic;

-- VME bootloader is inactive by default

  signal boot_en                    : std_logic := '0';
  signal boot_trig_p1, boot_exit_p1 : std_logic;
  signal CONTROL                    : std_logic_vector(35 downto 0);
  signal CLK                        : std_logic;
  signal TRIG0                      : std_logic_vector(31 downto 0);
  signal TRIG1                      : std_logic_vector(31 downto 0);
  signal TRIG2                      : std_logic_vector(31 downto 0);
  signal TRIG3                      : std_logic_vector(31 downto 0);

  signal boot_config_int                 : std_logic;
  signal erase_afpga_n, erase_afpga_n_d0 : std_logic;

  signal pllout_clk_fb_sys, pllout_clk_sys, clk_sys : std_logic;
  signal rst_n_sys                                  : std_logic;
  signal go_passive                                 : std_logic;
  signal vme_idle                                   : std_logic;

  signal pll_reset_count : unsigned(15 downto 0);

  signal spi_cs_n_int, spi_mosi_int, spi_sclk_int : std_logic;
  signal pass_flash: std_logic;

  signal vme_in : t_vme64x_in;
  signal vme_out_boot : t_vme64x_out;
  signal vme_out_bridge : t_vme64x_out;

  signal vme_data_b_out    : std_logic_vector(31 downto 0);
  signal vme_addr_b_out    : std_logic_vector(31 downto 1);
  signal vme_lword_n_b_out : std_logic;
  signal vme_data_dir_int  : std_logic;
  signal vme_addr_dir_int  : std_logic;
  signal vme_ga            : std_logic_vector(5 downto 0);
  signal vme_berr_n        : std_logic;
  signal vme_irq_n         : std_logic_vector(7 downto 1);

  signal addr_decoder_in  : std_logic_vector(31 downto 1);
  signal addr_decoder_out  : std_logic_vector(31 downto 1);
  signal decode_start  : std_logic;
  signal decode_done   : std_logic;
  signal r_am            : std_logic_vector(5 downto 0);
  signal decode_sel    : std_logic;
  signal cr_csr_addr   : std_logic_vector(18 downto 2);
  signal cr_csr_data_in   : std_logic_vector(7 downto 0);
  signal cr_csr_data_out   : std_logic_vector(7 downto 0);
  signal cr_csr_we     : std_logic;
  signal cr_csr_req    : std_logic;
  signal cr_csr_done   : std_logic;
  signal r_module_enable : std_logic;
  signal r_bar           : std_logic_vector(4 downto 0);
  signal r_int_level     : std_logic_vector(2 downto 0);
  signal r_int_vector    : std_logic_vector(7 downto 0);
  signal r_irq_pending   : std_logic;
  signal r_irq_ack       : std_logic;

  signal afpga_clk_0 : std_logic;
  signal afpga_clk_180 : std_logic;
  signal pllout_clk_fb_afpga : std_logic;
  signal pllout_clk_afpga_0 : std_logic;
  signal pllout_clk_afpga_180 : std_logic;

  signal afpga_dout      : std_logic_vector(15 downto 0);
  signal afpga_din       : std_logic_vector(15 downto 0);
  signal afpga_frame_out : std_logic;
  signal afpga_frame_in  : std_logic;

  signal clk_bridge : std_logic;
  signal rst_n_bridge : std_logic;

  signal cnt : unsigned(19 downto 0) := "00000000000000000000";

  signal clk2 : std_logic;

signal afpga_cso_b_int : std_logic;
signal afpga_cclk_int : std_logic;
  signal afpga_mosi_int : std_logic;
  signal boot_cclk : std_logic;

  signal heartbeat_tx, heartbeat_rx : std_logic;
begin

-- PLL for producing 83.3 MHz system clock (clk_sys) from a 20 MHz reference.
  U_Sys_clk_pll : PLL_BASE
    generic map (
      BANDWIDTH          => "OPTIMIZED",
      CLK_FEEDBACK       => "CLKFBOUT",
      COMPENSATION       => "INTERNAL",
      DIVCLK_DIVIDE      => 1,
      CLKFBOUT_MULT      => 50,
      CLKFBOUT_PHASE     => 0.000,
      CLKOUT0_DIVIDE     => 12,         -- 83.3 MHz
      CLKOUT0_PHASE      => 0.000,
      CLKOUT0_DUTY_CYCLE => 0.500,
      CLKOUT1_DIVIDE     => 16,         -- 62.5 MHz
      CLKOUT1_PHASE      => 0.000,
      CLKOUT1_DUTY_CYCLE => 0.500,
      CLKOUT2_DIVIDE     => 8,
      CLKOUT2_PHASE      => 0.000,
      CLKOUT2_DUTY_CYCLE => 0.500,
      CLKIN_PERIOD       => 50.0,
      REF_JITTER         => 0.016)
    port map (
      CLKFBOUT => pllout_clk_fb_sys,
      CLKOUT0  => pllout_clk_sys,
      CLKOUT1  => open,                 --pllout_clk_sys,
      CLKOUT2  => open,
      CLKOUT3  => open,
      CLKOUT4  => open,
      CLKOUT5  => open,
      LOCKED   => open,
      RST      => '0',
      CLKFBIN  => pllout_clk_fb_sys,
      CLKIN    => lclk_i);

  U_clk_sys_buf : BUFG
    port map (
      O => clk_sys,
      I => pllout_clk_sys);

  U_Powerup_Reset : reset_gen
    port map (
      clk_sys_i       => clk_sys,
      rst_vme_n_a_i   => vme_sysreset_n_i,
      rst_local_n_a_i => rst_n_i,
      rst_n_o         => rst_n_sys);

-------------------------------------------------------------------------------
-- Chipscope instantiation (for VME bus monitoring, I sincerely hate VMetro)
-------------------------------------------------------------------------------

  chipscope_ila_1 : chipscope_ila
    port map (
      CONTROL => CONTROL,
      CLK     => clk_bridge,
      TRIG0   => TRIG0,
      TRIG1   => TRIG1,
      TRIG2   => TRIG2,
      TRIG3   => TRIG3);

  chipscope_icon_1 : chipscope_icon
    port map (
      CONTROL0 => CONTROL);

  -- process(lclk_i)
  -- begin
  --   if rising_edge(lclk_i) then
  --     CNT <= cnt + 1;
  --   end if;
  -- end process;

  -- debugled_n_o(1) <= std_logic(cnt(19));

  
  
  -- trig0(0) <= rst_n_bridge;
  -- trig0(1) <= rst_n_i;
  -- trig0(2) <= rst_n_sys;
  -- trig0(7 downto 4) <= std_logic_vector(cnt);

   -- TRIG0(31 downto 1) <= VME_ADDR_b;
   -- TRIG1(31 downto 0) <= VME_DATA_b;
   -- TRIG2(5 downto 0)  <= VME_AM_i;
   -- trig2(7 downto 6)  <= VME_DS_n_i;
   -- trig2(12 downto 8) <= VME_GA_i;
   -- trig2(14)          <= VME_DTACK_n_o;
   -- trig2(15)          <= VME_DTACK_oe_o;
   -- trig2(16)          <= VME_LWORD_n_b;
   -- trig2(17)          <= VME_WRITE_n_i;
   -- trig2(18)          <= VME_AS_n_i;
   -- trig2(19)          <= VME_DATA_DIR_o;
   -- trig2(20)          <= VME_DATA_OE_N_o;
   -- trig2(21)          <= VME_addr_DIR_o;
   -- trig2(22)          <= VME_addr_OE_N_o;
   -- trig2(23)          <= rst_n_i;
   -- trig2(24)          <= '1';
   -- trig2(25)          <= vme_sysreset_n_i;
   -- trig2(26)          <= passive;
   -- trig2(27)          <= vme_idle;
   -- trig2(28)          <= rst_n_sys;

  -- gc_frequency_meter_1: entity work.gc_frequency_meter
  --   generic map (
  --     g_WITH_INTERNAL_TIMEBASE => true,
  --     g_CLK_SYS_FREQ           => 62500000,
  --     g_SYNC_OUT               => false,
  --     g_COUNTER_BITS           => 32)
  --   port map (
  --     clk_sys_i    => clk_bridge,
  --     clk_in_i     => clk_sys,
  --     rst_n_i      => rst_n_sys,
  --     pps_p1_i     => '0',
  --     freq_o       => trig3,
  --     freq_valid_o => trig0(8));

  -- trig2 <= x"deadbeef";
  
  


  xvme64x_bridge_serdes_1: entity work.xvme64x_bridge_serdes
    generic map (
      g_mode     => SYS_FPGA,
      g_platform => "spartan6",
      g_CLOCK_PERIOD => 16)
    port map (
      rst_n_i        => rst_n_i,

    bridge_clk2_o => clk2,
      bridge_clk_o =>   clk_bridge,
      bridge_rst_n_o => rst_n_bridge,
      bridge_d_i     => afpga_dout,
      bridge_frame_i => afpga_frame_out,
      bridge_d_o     => afpga_din,
      bridge_frame_o => afpga_frame_in,

      ser_clk_p_i    => afpga_clk_p_i,
      ser_clk_n_i    => afpga_clk_n_i,
      ser_tx_o       => afpga_tx_o,
      ser_rx_i       => afpga_rx_i);
  


  trig0(15 downto 0) <= afpga_dout;
  trig0(16) <= afpga_frame_out;
  
  trig1(15 downto 0) <= afpga_din;
  trig1(16) <= afpga_frame_in;

  trig2(0) <= heartbeat_tx;
  trig2(1) <= heartbeat_rx;
  
  
  xvme64x_core_sys_1: entity work.xvme64x_core_sys
    generic map (
      g_CLOCK_PERIOD => 16)
    port map (
      afpga_clk_i          => clk_bridge,
      afpga_rst_n_i        => rst_n_bridge,
      vme_i          => vme_in,
      vme_o          => vme_out_bridge,
      afpga_d_i     => afpga_din,
      afpga_frame_i => afpga_frame_in,
      afpga_d_o     => afpga_dout,
      afpga_frame_o => afpga_frame_out,
      heartbeat_rx_o => heartbeat_rx,
      heartbeat_tx_o => heartbeat_tx
      );

  debugled_n_o(1) <= heartbeat_rx;
  debugled_n_o(2) <= heartbeat_tx;
  
  
  vme_ga     <= vme_gap_i & vme_ga_i;

  vme_in.as_n      <= vme_as_n_i;
  vme_in.rst_n     <= vme_sysreset_n_i;
  vme_in.write_n   <= vme_write_n_i;
  vme_in.am        <= vme_am_i;
  vme_in.ds_n      <= vme_ds_n_i;
  vme_in.ga        <= vme_ga;
  vme_in.lword_n   <= vme_lword_n_b;
  vme_in.addr      <= vme_addr_b;
  vme_in.data      <= vme_data_b;
  vme_in.iack_n    <= vme_iack_n_i;
  vme_in.iackin_n  <= vme_iackin_n_i;


  U_MiniVME : entity work.xmini_vme
    generic map (
      g_user_csr_start => resize(x"70000", 21),
      g_user_csr_end   => resize(x"70020", 21))
    port map (
      clk_sys_i       => clk_sys,
      rst_n_i         => rst_n_sys,
      vme_i => vme_in,
      vme_o => vme_out_boot,
      master_o        => wb_vme_in,
      master_i        => wb_vme_out,
      idle_o          => vme_idle);

  U_Bootloader_Core : entity work.sfpga_bootloader
    generic map (
      g_interface_mode      => CLASSIC,
      g_address_granularity => BYTE,
      g_idr_value           => c_CSR_SIGNATURE_SVEC7,
      g_target_board => "SVEC7")
    port map (
      clk_sys_i       => clk_sys,
      rst_n_i         => rst_n_sys,
      wb_cyc_i        => wb_vme_in.cyc,
      wb_stb_i        => wb_vme_in.stb,
      wb_we_i         => wb_vme_in.we,
      wb_adr_i        => wb_vme_in.adr,
      wb_sel_i        => wb_vme_in.sel,
      wb_dat_i        => wb_vme_in.dat,
      wb_dat_o        => wb_vme_out.dat,
      wb_ack_o        => wb_vme_out.ack,
      wb_stall_o      => wb_vme_out.stall,
      xlx_cclk_o      => boot_cclk,
      xlx_din_o       => boot_din_o,
      xlx_program_b_o => boot_config_int,
      xlx_init_b_i    => boot_status_i,
      xlx_done_i      => boot_done_i,
      boot_trig_p1_o  => boot_trig_p1,
      boot_exit_p1_o  => boot_exit_p1,
      boot_en_i       => boot_en,
      spi_sys_cs_n_o      => spi_cs_n_int,
      spi_sys_sclk_o      => spi_sclk_int,
      spi_sys_mosi_o      => spi_mosi_int,
      spi_sys_miso_i      => spi_sys_miso_i,
      spi_app_cs_n_o      => afpga_cso_b_int,
      spi_app_sclk_o      => afpga_cclk_int,
      spi_app_mosi_o      => afpga_mosi_int,
      spi_app_miso_i      => afpga_miso_i);

  -- trig0(0) <= boot_cclk;
  -- trig0(1) <= boot_config_o;
  -- trig0(2) <= boot_status_i;
  -- trig0(3) <= boot_din_o;
  -- trig0(4) <= boot_done_i;
  -- trig0(7 downto 5) <= afpga_mode_ctrl_o;

  boot_cs_o <= '0';

  boot_clk_b <= boot_cclk when boot_en = '1' else 'Z';
  afpga_cclk_b <= afpga_cclk_int when boot_en = '1' else boot_clk_b;
  afpga_mosi_b <= afpga_mosi_int when boot_en = '1' else 'Z';
  afpga_cso_b_b <= afpga_cso_b_int when boot_en = '1' else 'Z';
  
  

  -- slave serial mode to keep the AFPGA SPI quiet when bootloader is active,
  -- otherwise - master serial. mode pins are inverted!
  afpga_mode_ctrl_o <= "000" when boot_en = '1' else "111"; 
  
  vme_tdo_oe_o <= '0';
  vme_tdo_o <= '0';
  
  -- produces a longer pulse on PROGRAM_B pin of the Application FPGA when
  -- the VME bootloader mode is activated
  U_Extend_Erase_Pulse : gc_extend_pulse
    generic map (
      g_width => 100)
    port map (
      clk_i      => clk_sys,
      rst_n_i    => rst_n_sys,
      pulse_i    => boot_trig_p1,
      extended_o => erase_afpga_n);

  -- Erase the application FPGA as soon as we have received a bootloader
  -- trigger command - this is to prevent two VME cores from working simultaneously
  -- on a single bus.
  boot_config_o <= boot_config_int and (not erase_afpga_n);

  p_enable_disable_bootloader : process(clk_sys)
  begin
    if rising_edge(clk_sys) then
      if(rst_n_sys = '0') then
        boot_en    <= '0';  -- VME bootloader is inactive after reset
        go_passive <= '0';
      else

        erase_afpga_n_d0 <= erase_afpga_n;

        -- VME activation occurs after erasing the AFPGA
        if(erase_afpga_n = '0' and erase_afpga_n_d0 = '1') then
          boot_en    <= '1';
          go_passive <= '0';
        elsif(boot_exit_p1 = '1') then
          go_passive <= '1';
        elsif (go_passive = '1' and vme_idle = '1') then
          go_passive <= '0';
          boot_en    <= '0';
        end if;
      end if;
    end if;
  end process;

  -- drive the PLL CE (powerup reset)

  pll_ce_o <= '1';
  
  
  -- p_reset_cdcm61004_pll : process(clk_sys)
  -- begin
  --   if rising_edge(clk_sys) then
  --     if rst_n_sys = '0' then
  --       pll_reset_count <= (others => '0');
  --       pll_ce_o        <= '0';
  --     else
  --       if(pll_reset_count = c_PLL_RESET_DURATION) then
  --         pll_ce_o <= '1';
  --       else
  --         pll_reset_count <= pll_reset_count + 1;
  --       end if;
  --     end if;
  --   end if;
  -- end process;

  -- multiplex flash access between the AFPGA and SFPGA bootloader (if the
  -- AFPGA is programmed, it's wired to the SPI flash).
  spi_sys_cs_n_o <= spi_cs_n_int;
  spi_sys_sclk_o <= spi_sclk_int;
  spi_sys_mosi_o <= spi_mosi_int;
--  afpga_flash_miso_o <= spi_miso_i;
  
  -- When the VME bootloader is not active, do NOT drive any outputs and sit quiet.
  passive <= not boot_en;

  process(vme_out_boot, vme_out_bridge, boot_en)
  begin
    if boot_en = '0' then
      vme_berr_n <= vme_out_bridge.berr_n;
      VME_DTACK_n_o <= vme_out_bridge.dtack_n;
      vme_retry_n_o <=      vme_out_bridge.retry_n;   
      vme_retry_oe_o <=      vme_out_bridge.retry_oe;
      vme_lword_n_b_out <= vme_out_bridge.lword_n;
      vme_data_b_out <=    vme_out_bridge.data;
      vme_addr_b_out <= vme_out_bridge.addr;
      vme_irq_n <= vme_out_bridge.irq_n;
      vme_iackout_n_o <= vme_out_bridge.iackout_n;
      VME_DTACK_oe_o <= vme_out_bridge.dtack_oe;
      vme_data_dir_int <= vme_out_bridge.data_dir;
      VME_DATA_OE_N_o <=      vme_out_bridge.data_oe_n;
      vme_addr_dir_int <=      vme_out_bridge.addr_dir;
      VME_addr_OE_N_o <=      vme_out_bridge.addr_oe_n;
    else
      vme_berr_n <= vme_out_boot.berr_n;
      VME_DTACK_n_o <= vme_out_boot.dtack_n;
      vme_retry_n_o <=      vme_out_boot.retry_n;   
      vme_retry_oe_o <=      vme_out_boot.retry_oe;
      vme_lword_n_b_out <= vme_out_boot.lword_n;
      vme_data_b_out <=    vme_out_boot.data;
      vme_addr_b_out <= vme_out_boot.addr;
      vme_irq_n <= vme_out_boot.irq_n;
      vme_iackout_n_o <= vme_out_boot.iackout_n;
      VME_DTACK_oe_o <= vme_out_boot.dtack_oe;
      vme_data_dir_int <= vme_out_boot.data_dir;
      VME_DATA_OE_N_o <=      vme_out_boot.data_oe_n;
      vme_addr_dir_int <=      vme_out_boot.addr_dir;
      VME_addr_OE_N_o <=      vme_out_boot.addr_oe_n;
    end if;
  end process;
    
  
  -- VME_ADDR_b <= (others => 'Z');

  -- VME_DTACK_n_o   <= VME_DTACK_n_int   when passive = '0'                                                          else 'Z';
  -- vme_dtack_oe_o  <= vme_dtack_oe_int  when passive = '0'                                                          else 'Z';
  -- VME_DATA_DIR_o  <= vme_data_dir_int  when passive = '0'                                                          else 'Z';
  -- VME_DATA_OE_N_o <= VME_DATA_OE_N_int when passive = '0'                                                          else 'Z';
  -- VME_DATA_b      <= VME_DATA_o_int    when (passive = '0' and VME_DATA_OE_N_int = '0' and vme_data_dir_int = '1') else (others => 'Z');
  -- VME_ADDR_OE_N_o <= '0'               when passive = '0'                                                          else 'Z';
  -- VME_ADDR_DIR_o  <= '0'               when passive = '0'                                                          else 'Z';
  -- VME_LWORD_n_b   <= 'Z';

  -- debugled_n_o(1) <= '1';
  -- debugled_n_o(2) <= not boot_en;

  vme_berr_o <= '0'; --not vme_berr_n;
  vme_irq_o  <= (others => '0'); --not vme_irq_n;

  -- VME tri-state buffers
  vme_data_b <= vme_data_b_out when vme_data_dir_int = '1' else (others => 'Z');
  vme_addr_b <= vme_addr_b_out when vme_addr_dir_int = '1' else (others => 'Z');
  vme_lword_n_b <= vme_lword_n_b_out when vme_addr_dir_int = '1' else 'Z';

  vme_addr_dir_o <= vme_addr_dir_int;
  vme_data_dir_o <= vme_data_dir_int;

end rtl;


